/// <reference types="node" />
import { PacketWriter, PacketReader } from "..";
import * as nbt from "nbt-ts";
declare abstract class Packet {
    constructor(reader: PacketReader);
    abstract encode(writer: PacketWriter): void;
}
export interface Position {
    x: number;
    y: number;
    z: number;
}
export interface Slot {
    id: number;
    count: number;
    damage: number;
    nbt: nbt.Tag | null;
}
export declare type MetadataEntry = {
    index: number;
} & ({
    type: 0 | 1 | 2;
    value: number;
} | {
    type: 3;
    value: string;
} | {
    type: 4;
    value: any;
} | {
    type: 5;
    value: Slot | null;
} | {
    type: 6;
    value: boolean;
} | {
    type: 7;
    value: Position;
} | {
    type: 8;
    value: Position;
} | {
    type: 9;
    value: Position | null;
} | {
    type: 10;
    value: number;
} | {
    type: 11;
    value: Buffer | null;
} | {
    type: 12;
    value: number;
} | {
    type: 13;
    value: nbt.Tag;
});
export declare class SpawnObject extends Packet {
    eid: number;
    uuid: Buffer;
    type: number;
    x: number;
    y: number;
    z: number;
    yaw: number;
    pitch: number;
    data: number;
    vx: number;
    vy: number;
    vz: number;
    constructor(reader: PacketReader);
    encode(writer: PacketWriter): PacketWriter;
}
export declare class SpawnXPOrb extends Packet {
    eid: number;
    x: number;
    y: number;
    z: number;
    count: number;
    constructor(packet: PacketReader);
    encode(writer: PacketWriter): PacketWriter;
}
export declare class SpawnGlobalEntity extends Packet {
    eid: number;
    type: number;
    x: number;
    y: number;
    z: number;
    constructor(packet: PacketReader);
    encode(writer: PacketWriter): PacketWriter;
}
export declare class SpawnMob extends Packet {
    eid: number;
    uuid: Buffer;
    type: number;
    x: number;
    y: number;
    z: number;
    yaw: number;
    pitch: number;
    headPitch: number;
    data: number;
    vx: number;
    vy: number;
    vz: number;
    metadata: MetadataEntry[];
    constructor(reader: PacketReader);
    encode(writer: PacketWriter): Buffer;
}
export declare class SpawnPainting extends Packet {
    eid: number;
    uuid: Buffer;
    title: string;
    location: Position;
    direction: number;
    constructor(reader: PacketReader);
    encode(writer: PacketWriter): PacketWriter;
}
export declare class SpawnPlayer extends Packet {
    eid: number;
    uuid: Buffer;
    x: number;
    y: number;
    z: number;
    yaw: number;
    pitch: number;
    metadata: MetadataEntry[];
    constructor(reader: PacketReader);
    encode(writer: PacketWriter): PacketWriter;
}
export declare class Animation extends Packet {
    eid: number;
    id: number;
    constructor(reader: PacketReader);
    encode(writer: PacketWriter): void;
}
export {};
