"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Packet {
    constructor(reader) { }
}
function readMetadata(reader) {
    const metadata = [];
    while (true) {
        const index = reader.readInt8();
        if (index == -1)
            break;
        const type = reader.readVarInt();
        switch (type) {
            case 0:
                metadata.push({ index, type, value: reader.readInt8() });
                break;
            case 1:
                metadata.push({ index, type, value: reader.readVarInt() });
                break;
            case 2:
                metadata.push({ index, type, value: reader.readFloat() });
                break;
            case 3:
                metadata.push({ index, type, value: reader.readString() });
                break;
            case 4:
                metadata.push({ index, type, value: reader.readJSON() });
                break;
            case 5:
                {
                    const id = reader.readInt16();
                    metadata.push({
                        index, type, value: id == -1
                            ? { id, count: reader.readInt8(), damage: reader.readInt16(), nbt: reader.readNBT()?.value }
                            : null
                    });
                }
                ;
                break;
            case 6:
                metadata.push({ index, type, value: reader.readBool() });
                break;
            case 7:
                metadata.push({ index, type, value: reader.readPosition() });
                break;
            case 8:
                metadata.push({ index, type, value: reader.readPosition() });
                break;
            case 9:
                metadata.push({ index, type, value: reader.readBool() ? reader.readPosition() : null });
                break;
            case 10:
                metadata.push({ index, type, value: reader.readVarInt() });
                break;
            case 11:
                metadata.push({ index, type, value: reader.readBool() ? reader.read(16) : null });
                break;
            case 12:
                metadata.push({ index, type, value: reader.readVarInt() });
                break;
            case 13:
                metadata.push({ index, type, value: reader.readNBT()?.value });
                break;
            default: throw new Error(`Unexpected metadata type '${type}'`);
        }
    }
    return metadata;
}
function writeMetadata(writer, metadata) {
    for (const entry of metadata) {
        writer.writeUInt8(entry.index);
        writer.writeVarInt(entry.type);
        switch (entry.type) {
            case 0:
                writer.writeInt8(entry.value);
                break;
            case 1:
                writer.writeVarInt(entry.value);
                break;
            case 2:
                writer.writeFloat(entry.value);
                break;
            case 3:
                writer.writeString(entry.value);
                break;
            case 4:
                writer.writeJSON(entry.value);
                break;
            case 5:
                {
                    if (!entry.value) {
                        writer.writeInt16(-1);
                        break;
                    }
                    writer.writeInt16(entry.value.id).writeInt8(entry.value.count).writeInt16(entry.value.damage);
                    writer.writeNBT("", entry.value.nbt);
                }
                ;
                break;
            case 6:
                writer.writeBool(entry.value);
                break;
            case 7:
                writer.writePosition(entry.value);
                break;
            case 8:
                writer.writePosition(entry.value);
                break;
            case 9:
                writer.writeBool(entry.value != null),
                    entry.value && writer.writePosition(entry.value);
                break;
            case 10:
                writer.writeVarInt(entry.value);
                break;
            case 11:
                writer.writeBool(entry.value != null),
                    entry.value && writer.write(entry.value);
                break;
            case 12:
                writer.writeVarInt(entry.value);
                break;
            case 13:
                writer.writeNBT("", entry.value);
                break;
        }
    }
    writer.writeInt8(-1);
    return writer;
}
class SpawnObject extends Packet {
    constructor(reader) {
        super(reader);
        this.eid = reader.readVarInt();
        this.uuid = reader.read(16);
        this.type = reader.readInt8();
        this.x = reader.readDouble();
        this.y = reader.readDouble();
        this.z = reader.readDouble();
        this.yaw = reader.readInt8();
        this.pitch = reader.readInt8();
        this.data = reader.readInt32();
        this.vx = reader.readInt16();
        this.vy = reader.readInt16();
        this.vz = reader.readInt16();
    }
    encode(writer) {
        return writer
            .writeVarInt(this.eid).write(this.uuid).writeInt8(this.type)
            .writeDouble(this.x).writeDouble(this.y).writeDouble(this.z)
            .writeInt8(this.pitch).writeInt8(this.yaw).writeInt32(this.data)
            .writeInt16(this.vx).writeInt16(this.vy).writeInt16(this.vz);
    }
}
exports.SpawnObject = SpawnObject;
class SpawnXPOrb extends Packet {
    constructor(packet) {
        super(packet);
        this.eid = packet.readVarInt();
        this.x = packet.readDouble();
        this.y = packet.readDouble();
        this.z = packet.readDouble();
        this.count = packet.readInt16();
    }
    encode(writer) {
        return writer.writeVarInt(this.eid)
            .writeDouble(this.x).writeDouble(this.y).writeDouble(this.z)
            .writeInt16(this.count);
    }
}
exports.SpawnXPOrb = SpawnXPOrb;
class SpawnGlobalEntity extends Packet {
    constructor(packet) {
        super(packet);
        this.eid = packet.readVarInt();
        this.type = packet.readInt8();
        this.x = packet.readDouble();
        this.y = packet.readDouble();
        this.z = packet.readDouble();
    }
    encode(writer) {
        return writer.writeVarInt(this.eid).writeInt8(this.type)
            .writeDouble(this.x).writeDouble(this.y).writeDouble(this.z);
    }
}
exports.SpawnGlobalEntity = SpawnGlobalEntity;
class SpawnMob extends Packet {
    constructor(reader) {
        super(reader);
        this.metadata = [];
        this.eid = reader.readVarInt();
        this.uuid = reader.read(16);
        this.type = reader.readVarInt();
        this.x = reader.readDouble();
        this.y = reader.readDouble();
        this.z = reader.readDouble();
        this.yaw = reader.readInt8();
        this.pitch = reader.readInt8();
        this.headPitch = reader.readInt8();
        this.data = reader.readInt32();
        this.vx = reader.readInt16();
        this.vy = reader.readInt16();
        this.vz = reader.readInt16();
        this.metadata = readMetadata(reader);
    }
    encode(writer) {
        writer
            .writeVarInt(this.eid).write(this.uuid).writeInt8(this.type)
            .writeDouble(this.x).writeDouble(this.y).writeDouble(this.z)
            .writeInt8(this.pitch).writeInt8(this.yaw).writeInt8(this.headPitch)
            .writeInt32(this.data)
            .writeInt16(this.vx).writeInt16(this.vy).writeInt16(this.vz);
        return writeMetadata(writer, this.metadata).encode();
    }
}
exports.SpawnMob = SpawnMob;
class SpawnPainting extends Packet {
    constructor(reader) {
        super(reader);
        this.eid = reader.readVarInt();
        this.uuid = reader.read(16);
        this.title = reader.readString();
        this.location = reader.readPosition();
        this.direction = reader.readInt8();
    }
    encode(writer) {
        return writer.writeVarInt(this.eid).write(this.uuid)
            .writeString(this.title)
            .writePosition(this.location)
            .writeInt8(this.direction);
    }
}
exports.SpawnPainting = SpawnPainting;
class SpawnPlayer extends Packet {
    constructor(reader) {
        super(reader);
        this.eid = reader.readVarInt();
        this.uuid = reader.read(16);
        this.x = reader.readDouble();
        this.y = reader.readDouble();
        this.z = reader.readDouble();
        this.yaw = reader.readInt8();
        this.pitch = reader.readInt8();
        this.metadata = readMetadata(reader);
    }
    encode(writer) {
        writer.writeVarInt(this.eid).write(this.uuid)
            .writeDouble(this.x).writeDouble(this.y).writeDouble(this.z)
            .writeInt8(this.pitch).writeInt8(this.yaw);
        return writeMetadata(writer, this.metadata);
    }
}
exports.SpawnPlayer = SpawnPlayer;
class Animation extends Packet {
    constructor(reader) {
        super(reader);
        this.eid = reader.readVarInt();
        this.id = reader.readUInt8();
    }
    encode(writer) {
    }
}
exports.Animation = Animation;
